# Webhook Event Reference

| Event | Description | Permissions needed | Payload
| :--- | :--- | :--- | :--- |
|`checkout.customer.before.login` | Triggers as soon as a customer logs in | - | {"email":"string"}
|`checkout.customer.changed-payment-method` | Triggers when a customer changes his payment method in the checkout process | `customer:read` | {"entity":"customer"}
|`checkout.customer.deleted` | Triggers if a customer gets deleted | `customer:read` | {"entity":"customer"}
|`checkout.customer.double_opt_in_guest_order` | Triggers as soon as double opt-in is accepted in a guest order | `customer:read` | {"entity":"customer","confirmUrl":"string"}
|`checkout.customer.double_opt_in_registration` | Triggers when a customer commits to his registration via double opt in | `customer:read` | {"entity":"customer","confirmUrl":"string"}
|`checkout.customer.guest_register` | __EMPTY__ | `customer:read` | {"entity":"customer"}
|`checkout.customer.login` | Triggers as soon as a customer logs in | `customer:read` | {"entity":"customer","contextToken":"string"}
|`checkout.customer.logout` | Triggers when a customer logs out | `customer:read` | {"entity":"customer"}
|`checkout.customer.register` | Triggers when a new customer was registered | `customer:read` | {"entity":"customer"}
|`checkout.order.payment_method.changed` | __EMPTY__ | `order:read` `order_transaction:read` | {"entity":"order_transaction"}
|`checkout.order.placed` | Triggers when an order is placed | `order:read` | {"entity":"order"}
|`contact_form.send` | Triggers when a contact form is send | - | {"contactFormData":"object"}
|`customer.group.registration.accepted` | __EMPTY__ | `customer:read` `customer_group:read` | {"entity":"customer_group"}
|`customer.group.registration.declined` | __EMPTY__ | `customer:read` `customer_group:read` | {"entity":"customer_group"}
|`customer.recovery.request` | Triggers when a customer recovers his password | `customer_recovery:read` `customer:read` | {"entity":"customer","resetUrl":"string","shopName":"string"}
|`mail.after.create.message` | __EMPTY__ | - | {"data":"array","message":"object"}
|`mail.before.send` | Triggers before a mail is send | - | {"data":"array","templateData":"array"}
|`mail.sent` | Triggers when a mail is send from Shopware | - | {"subject":"string","contents":"string","recipients":"array"}
|`newsletter.confirm` | __EMPTY__ | `newsletter_recipient:read` | {"entity":"newsletter_recipient"}
|`newsletter.register` | __EMPTY__ | `newsletter_recipient:read` | {"entity":"newsletter_recipient","url":"string"}
|`newsletter.unsubscribe` | __EMPTY__ | `newsletter_recipient:read` | {"entity":"newsletter_recipient"}
|`newsletter.update` | __EMPTY__ | `newsletter_recipient:read` | {"entity":"newsletter_recipient"}
|`product_export.log` | __EMPTY__ | - | {"name":"string"}
|`state_enter.order.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.returned` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.returned_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.shipped` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_delivery.state.shipped_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.authorized` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.chargeback` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.paid` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.paid_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.refunded` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.refunded_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.reminded` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction.state.unconfirmed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture.state.pending` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture_refund.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture_refund.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture_refund.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture_refund.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_enter.order_transaction_capture_refund.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.returned` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.returned_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.shipped` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_delivery.state.shipped_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.authorized` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.chargeback` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.paid` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.paid_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.refunded` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.refunded_partially` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.reminded` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction.state.unconfirmed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture.state.pending` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture_refund.state.cancelled` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture_refund.state.completed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture_refund.state.failed` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture_refund.state.in_progress` | __EMPTY__ | `order:read` | {"entity":"order"}
|`state_leave.order_transaction_capture_refund.state.open` | __EMPTY__ | `order:read` | {"entity":"order"}
|`user.recovery.request` | __EMPTY__ | `user_recovery:read` | {"entity":"user_recovery","resetUrl":"string"}
|`product.written` | Triggers when a product is written | `product:read` | {"entity":"product","operation":true,"primaryKey":"array string","payload":"array"}
|`product.deleted` | Triggers when a product is deleted | `product:read` | {"entity":"product","operation":"update insert","primaryKey":"array string","payload":"array"}
|`product_price.written` | Triggers when a product_price is written | `product_price:read` | {"entity":"product_price","operation":true,"primaryKey":"array string","payload":"array"}
|`product_price.deleted` | Triggers when a product_price is deleted | `product_price:read` | {"entity":"product_price","operation":"update insert","primaryKey":"array string","payload":"array"}
|`category.written` | Triggers when a category is written | `category:read` | {"entity":"category","operation":true,"primaryKey":"array string","payload":"array"}
|`category.deleted` | Triggers when a category is deleted | `category:read` | {"entity":"category","operation":"update insert","primaryKey":"array string","payload":"array"}
|`sales_channel.written` | Triggers when a sales_channel is written | `sales_channel:read` | {"entity":"sales_channel","operation":true,"primaryKey":"array string","payload":"array"}
|`sales_channel.deleted` | Triggers when a sales_channel is deleted | `sales_channel:read` | {"entity":"sales_channel","operation":"update insert","primaryKey":"array string","payload":"array"}
|`sales_channel_domain.written` | Triggers when a sales_channel_domain is written | `sales_channel_domain:read` | {"entity":"sales_channel_domain","operation":true,"primaryKey":"array string","payload":"array"}
|`sales_channel_domain.deleted` | Triggers when a sales_channel_domain is deleted | `sales_channel_domain:read` | {"entity":"sales_channel_domain","operation":"update insert","primaryKey":"array string","payload":"array"}
|`customer.written` | Triggers when a customer is written | `customer:read` | {"entity":"customer","operation":true,"primaryKey":"array string","payload":"array"}
|`customer.deleted` | Triggers when a customer is deleted | `customer:read` | {"entity":"customer","operation":"update insert","primaryKey":"array string","payload":"array"}
|`customer_address.written` | Triggers when a customer_address is written | `customer_address:read` | {"entity":"customer_address","operation":true,"primaryKey":"array string","payload":"array"}
|`customer_address.deleted` | Triggers when a customer_address is deleted | `customer_address:read` | {"entity":"customer_address","operation":"update insert","primaryKey":"array string","payload":"array"}
|`order.written` | Triggers when a order is written | `order:read` | {"entity":"order","operation":true,"primaryKey":"array string","payload":"array"}
|`order.deleted` | Triggers when a order is deleted | `order:read` | {"entity":"order","operation":"update insert","primaryKey":"array string","payload":"array"}
|`order_address.written` | Triggers when a order_address is written | `order_address:read` | {"entity":"order_address","operation":true,"primaryKey":"array string","payload":"array"}
|`order_address.deleted` | Triggers when a order_address is deleted | `order_address:read` | {"entity":"order_address","operation":"update insert","primaryKey":"array string","payload":"array"}
|`document.written` | Triggers when a document is written | `document:read` | {"entity":"document","operation":true,"primaryKey":"array string","payload":"array"}
|`document.deleted` | Triggers when a document is deleted | `document:read` | {"entity":"document","operation":"update insert","primaryKey":"array string","payload":"array"}
